from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.namedfile.file import NamedFile
from Products.CMFDiffTool import BinaryDiff
from Products.CMFDiffTool import namedfile
from Products.CMFDiffTool import testing
from Products.CMFDiffTool.interfaces import IDifference
from Products.CMFDiffTool.tests.BaseTestCase import BaseDXTestCase


class BinaryDiffTestCase(BaseDXTestCase):
    def test_should_detect_different_filename(self):
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.portal.invokeFactory(
            testing.TEST_CONTENT_TYPE_ID,
            "obj1",
            file=NamedFile(data="contents", filename="blah.txt"),
        )
        obj1 = self.portal["obj1"]

        self.portal.invokeFactory(
            testing.TEST_CONTENT_TYPE_ID,
            "obj2",
            file=NamedFile(data="contents", filename="bleh.txt"),
        )
        obj2 = self.portal["obj2"]

        diff = namedfile.NamedFileBinaryDiff(obj1, obj2, "file")
        self.assertTrue(IDifference.providedBy(diff))
        self.assertFalse(diff.same)

    def test_should_detect_different_data(self):
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.portal.invokeFactory(
            testing.TEST_CONTENT_TYPE_ID,
            "obj1",
            file=NamedFile(data="contents", filename="f.txt"),
        )
        obj1 = self.portal["obj1"]

        self.portal.invokeFactory(
            testing.TEST_CONTENT_TYPE_ID,
            "obj2",
            file=NamedFile(data="different contents", filename="f.txt"),
        )
        obj2 = self.portal["obj2"]

        diff = namedfile.NamedFileBinaryDiff(obj1, obj2, "file")
        self.assertTrue(IDifference.providedBy(diff))
        self.assertFalse(diff.same)

    def test_should_detect_same_data_and_filename(self):
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.portal.invokeFactory(
            testing.TEST_CONTENT_TYPE_ID,
            "obj1",
            file=NamedFile(data="contents", filename="f.txt"),
        )
        obj1 = self.portal["obj1"]

        self.portal.invokeFactory(
            testing.TEST_CONTENT_TYPE_ID,
            "obj2",
            file=NamedFile(data="contents", filename="f.txt"),
        )
        obj2 = self.portal["obj2"]

        diff = namedfile.NamedFileBinaryDiff(obj1, obj2, "file")
        self.assertTrue(IDifference.providedBy(diff))
        self.assertTrue(diff.same)

    def test_should_escape_html(self):
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.portal.invokeFactory(
            testing.TEST_CONTENT_TYPE_ID,
            "obj1",
            file=NamedFile(data="contents", filename="blah.txt"),
        )
        obj1 = self.portal["obj1"]

        self.portal.invokeFactory(
            testing.TEST_CONTENT_TYPE_ID,
            "obj2",
            file=NamedFile(
                data='<script>alert("Hacker data")</script>',
                filename='<script>alert("Hacker filename")</script>.txt',
            ),
        )
        obj2 = self.portal["obj2"]

        diff = namedfile.NamedFileBinaryDiff(obj1, obj2, "file")
        self.assertTrue(IDifference.providedBy(diff))
        self.assertFalse(diff.same)
        # The script tag should be escaped.
        self.assertNotIn("<script", diff.inline_diff())
        self.assertIn("&gt;", diff.inline_diff())

        # Test the more basic BinaryDiff.
        # It only compares the file names.
        # It uses the 'getFilename' method of the file,
        # which namedfiles do not have.  So we hack it.
        obj1.file.getFilename = lambda: obj1.file.filename
        obj2.file.getFilename = lambda: obj2.file.filename
        diff = BinaryDiff.BinaryDiff(obj1, obj2, "file")
        self.assertTrue(IDifference.providedBy(diff))
        self.assertFalse(diff.same)
        # The script tag should be escaped.
        self.assertNotIn("<script", diff.inline_diff())
        self.assertIn("&gt;", diff.inline_diff())



from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.testing import FunctionalTesting
from plone.app.testing import PloneSandboxLayer
from plone.dexterity.fti import DexterityFTI
from Products.CMFCore.utils import getToolByName
from zope.component import getSiteManager
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


TEST_CONTENT_TYPE_ID = "TestContentType"

VOCABULARY_TUPLES = [
    ("first_value", "First Title"),
    ("second_value", None),
    ("third_value", "Third Title"),
]

VOCABULARY = SimpleVocabulary(
    [SimpleTerm(value=v, title=t) for (v, t) in VOCABULARY_TUPLES]
)


def vocabulary_factory(context):
    return VOCABULARY


class DXLayer(PloneSandboxLayer):
    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpPloneSite(self, portal):
        """Set up additional products and ZCML required to test
        this product.
        """
        # setup dexterity
        types_tool = getToolByName(portal, "portal_types")

        sm = getSiteManager(portal)
        sm.registerUtility(
            component=vocabulary_factory,
            provided=IVocabularyFactory,
            name="Products.CMFDiffTool.testing.VOCABULARY",
        )

        fti = DexterityFTI(
            TEST_CONTENT_TYPE_ID,
            factory=TEST_CONTENT_TYPE_ID,
            global_allow=True,
            behaviors=(
                "plone.app.versioningbehavior.behaviors.IVersionable",
                "plone.app.dexterity.behaviors.metadata.IBasic",
                "plone.app.relationfield.behavior.IRelatedItems",
                "plone.app.contenttypes.behaviors.collection.ICollection",
            ),
            model_source="""
            <model xmlns='http://namespaces.plone.org/supermodel/schema'>
                <schema>
                    <field name='text' type='zope.schema.Text'>
                        <title>Text</title>
                        <required>False</required>
                    </field>
                    <field name='file' type='plone.namedfile.field.NamedFile'>
                        <title>File</title>
                        <required>False</required>
                    </field>
                    <field name='date' type='zope.schema.Date'>
                        <title>Date</title>
                        <required>False</required>
                    </field>
                    <field name='files' type='zope.schema.List'>
                        <title>Date</title>
                        <required>False</required>
                        <value_type type='plone.namedfile.field.NamedFile'>
                            <title>Val</title>
                        </value_type>
                    </field>
                    <field name='choice' type='zope.schema.Choice'>
                        <title>Choice</title>
                        <required>False</required>
                        <vocabulary>Products.CMFDiffTool.testing.VOCABULARY</vocabulary>
                    </field>
                    <field name="choices" type="zope.schema.List">
                        <title>Choices</title>
                        <required>False</required>
                        <value_type type="zope.schema.Choice">
                            <vocabulary>Products.CMFDiffTool.testing.VOCABULARY</vocabulary>
                        </value_type>
                    </field>
                </schema>
            </model>
            """,
        )
        types_tool._setObject(TEST_CONTENT_TYPE_ID, fti)


PACKAGE_DX_FIXTURE = DXLayer()

CMFDiffToolDXLayer = FunctionalTesting(
    bases=(PACKAGE_DX_FIXTURE,), name="Products.CMFDiffTool.DX:functional"
)



from html import escape
from Products.PortalTransforms.transforms.safe_html import SafeHTML


def safe_unicode(value):
    if isinstance(value, str):
        return value
    try:
        value = str(value)
    except UnicodeDecodeError:
        value = value.decode("utf-8", "replace")
    return value


def safe_utf8(value):
    return safe_unicode(value).encode("utf-8")


def scrub_html(value):
    # Strip illegal HTML tags from string text.
    transform = SafeHTML()
    return transform.scrub_html(value)


# We will have two functions:
# - html_escape: escape html, for example turn '<' into '&lt;'
# - html_safe: return html with dangerous tags removed, using safe html transform.
#
# The convert function that we use in safe_html
# cannot handle a non string-like value, for example an integer.
# Same is true for the escape function.
# Seems good to always return a string-like value though.
# This gets inserted in a string/text.


def html_escape(value):
    value = safe_unicode(value)
    return escape(value, 1)


def html_safe(value):
    value = safe_unicode(value)
    return scrub_html(value)



from Products.CMFCore.utils import getToolByName
from Products.CMFDiffTool.interfaces import IDiffTool
from Products.GenericSetup.utils import exportObjects
from Products.GenericSetup.utils import importObjects
from Products.GenericSetup.utils import XMLAdapterBase
from zope.interface import implementer


@implementer(IDiffTool)
class DiffToolXMLAdapter(XMLAdapterBase):
    """In- and exporter for DiffTool."""

    name = "diff_tool"

    def _exportNode(self):
        """Export the object as a DOM node."""
        node = self._doc.createElement("object")
        node.appendChild(self._extractDiffToolSettings())

        self._logger.info("DiffTool settings exported.")
        return node

    def _importNode(self, node):
        if self.environ.shouldPurge():
            self._purgeDiffToolSettings()

        self._initDiffToolSettings(node)
        self._logger.info("DiffTool settings imported.")

    def _purgeDiffToolSettings(self):
        self.context.manage_editDiffFields({})

    def _initDiffToolSettings(self, node):
        for child in node.childNodes:
            if child.nodeName == "difftypes":
                for type_entry in child.getElementsByTagName("type"):
                    ptype = type_entry.getAttribute("portal_type")
                    fields = {}
                    for field in type_entry.getElementsByTagName("field"):
                        name = field.getAttribute("name")
                        diff = field.getAttribute("difftype")
                        fields[name] = diff
                        self.context.setDiffForPortalType(ptype, fields)

    def _extractDiffToolSettings(self):
        node = self._doc.createElement("difftypes")
        ttool = getToolByName(self.context, "portal_types")
        for ptype in ttool.listContentTypes():
            diffs = self.context.getDiffForPortalType(ptype)
            if diffs:
                child = self._doc.createElement("type")
                child.setAttribute("portal_type", ptype)
                node.appendChild(child)
            for field_name, diff in diffs.items():
                field = self._doc.createElement("field")
                field.setAttribute("name", field_name)
                field.setAttribute("difftype", diff)
                child.appendChild(field)
        return node


def importDiffTool(context):
    """Import Factory Tool configuration."""
    site = context.getSite()
    tool = getToolByName(site, "portal_diff", None)

    if tool is not None:
        importObjects(tool, "", context)


def exportDiffTool(context):
    """Export Factory Tool configuration."""
    site = context.getSite()
    tool = getToolByName(site, "portal_diff", None)
    if tool is None:
        logger = context.getLogger("difftool")
        logger.info("Nothing to export.")
        return

    exportObjects(tool, "", context)



from AccessControl.class_init import InitializeClass
from plone.dexterity.interfaces import IDexterityContent
from Products.CMFDiffTool.choicediff import get_field_object
from Products.CMFDiffTool.choicediff import title_or_value
from Products.CMFDiffTool.FieldDiff import FieldDiff
from Products.CMFDiffTool.utils import html_escape


class ListDiff(FieldDiff):
    """Text difference"""

    meta_type = "List Diff"

    def __init__(
        self,
        obj1,
        obj2,
        field,
        id1=None,
        id2=None,
        field_name=None,
        field_label=None,
        schemata=None,
    ):
        FieldDiff.__init__(
            self, obj1, obj2, field, id1, id2, field_name, field_label, schemata
        )
        self._vocabulary = None

        # Tries to find a vocabulary. First we need to find an object and
        # the field instance.
        obj = obj1 if (obj1 is not None) else obj2
        field_name = field_name or field
        if obj and field_name and IDexterityContent.providedBy(obj):
            field_instance = get_field_object(obj, field_name)
            if field_instance is not None:
                # Binding the field to an object will construct the vocabulary
                # using a factory if necessary.
                try:
                    self._vocabulary = field_instance.value_type.bind(obj).vocabulary
                except Exception:
                    pass

    def chk_hashable(self, value):
        if self._vocabulary is not None:
            value = title_or_value(self._vocabulary, value)
        try:
            hash(value)
        except TypeError as e:
            value = repr(e) + ": " + repr(value)
        return value

    def _parseField(self, value, filename=None):
        """Parse a field value in preparation for diffing"""
        if type(value) is list or type(value) is tuple:
            values = []
            for v in value:
                values.append(self.chk_hashable(v))
            return values
        else:
            if type(value) is set:
                return list(value)
            else:
                return [self.chk_hashable(value)]


class RelationListDiff(FieldDiff):
    meta_type = "Related List Diff"

    same_fmt = """<div class="%s"><a target="_blank" href="%s">%s</a></div>"""
    inlinediff_fmt = """<div class="%s">
        <div class="%s"><a target="_blank" href="%s">%s</a></div>
    </div>"""

    def _parseField(self, value, filename=None):
        """Take RelationValues and just return the target UID
        so we can compare"""

        if filename is None:
            # Since we only want to compare a single field, make a
            # one-item list out of it
            return ["/".join(val.getPhysicalPath()) for val in value]
        else:
            return [
                self.filenameTitle(filename),
                ["/".join(val.getPhysicalPath()) for val in value],
            ]

    def inline_diff(self):
        css_class = "InlineDiff"
        inlinediff_fmt = self.inlinediff_fmt
        same_fmt = self.same_fmt
        r = []
        for tag, alo, ahi, blo, bhi in self.getLineDiffs():
            if tag == "replace":
                for i in range(alo, ahi):
                    obj = self.oldValue[i]
                    obj_title = html_escape(obj.Title())
                    obj_url = obj.absolute_url()
                    r.append(
                        inlinediff_fmt % (css_class, "diff_sub", obj_url, obj_title)
                    )
                for i in range(blo, bhi):
                    obj = self.newValue[i]
                    obj_title = html_escape(obj.Title())
                    obj_url = obj.absolute_url()
                    r.append(
                        inlinediff_fmt % (css_class, "diff_add", obj_url, obj_title)
                    )
            elif tag == "delete":
                for i in range(alo, ahi):
                    obj = self.oldValue[i]
                    obj_title = html_escape(obj.Title())
                    obj_url = obj.absolute_url()
                    r.append(
                        inlinediff_fmt % (css_class, "diff_sub", obj_url, obj_title)
                    )
            elif tag == "insert":
                for i in range(blo, bhi):
                    obj = self.newValue[i]
                    obj_title = html_escape(obj.Title())
                    obj_url = obj.absolute_url()
                    r.append(
                        inlinediff_fmt % (css_class, "diff_add", obj_url, obj_title)
                    )
            elif tag == "equal":
                for i in range(alo, ahi):
                    obj = self.oldValue[i]
                    obj_title = html_escape(obj.Title())
                    obj_url = obj.absolute_url()
                    r.append(same_fmt % (css_class, obj_url, obj_title))
            else:
                raise ValueError("unknown tag %s" % tag)
        return "\n".join(r)

    def ndiff(self):
        """Return a textual diff"""
        r = []
        for tag, alo, ahi, blo, bhi in self.getLineDiffs():
            if tag == "replace":
                for i in range(alo, ahi):
                    obj = self.oldValue[i]
                    obj_url = obj.absolute_url()
                    r.append("- %s" % obj_url)
                for i in range(blo, bhi):
                    obj = self.newValue[i]
                    obj_url = obj.absolute_url()
                    r.append("+ %s" % obj_url)
            elif tag == "delete":
                for i in range(alo, ahi):
                    obj = self.oldValue[i]
                    obj_url = obj.absolute_url()
                    r.append("- %s" % obj_url)
            elif tag == "insert":
                for i in range(blo, bhi):
                    obj = self.newValue[i]
                    obj_url = obj.absolute_url()
                    r.append("+ %s" % obj_url)
            elif tag == "equal":
                for i in range(alo, ahi):
                    obj = self.oldValue[i]
                    obj_url = obj.absolute_url()
                    r.append("  %s" % obj_url)
            else:
                raise ValueError("unknown tag %r", tag)
        return "\n".join(r)


InitializeClass(ListDiff)
InitializeClass(RelationListDiff)



from AccessControl.class_init import InitializeClass
from Products.CMFDiffTool.BaseDiff import _getValue
from Products.CMFDiffTool.BaseDiff import BaseDiff
from Products.CMFDiffTool.utils import html_escape

import difflib


class FieldDiff(BaseDiff):
    """Text difference"""

    meta_type = "Field Diff"

    same_fmt = """<div class="%s">%s</div>"""
    inlinediff_fmt = """<div class="%s">
    <div class="diff_sub">%s</div>
    <div class="diff_add">%s</div>
</div>"""

    def _parseField(self, value, filename=None):
        """Parse a field value in preparation for diffing"""
        if filename is None:
            # Since we only want to compare a single field, make a
            # one-item list out of it
            return [value]
        else:
            return [
                self.filenameTitle(filename),
                value,
            ]

    def getLineDiffs(self):
        a = self._parseField(self.oldValue, filename=self.oldFilename)
        b = self._parseField(self.newValue, filename=self.newFilename)
        return difflib.SequenceMatcher(None, a, b).get_opcodes()

    def testChanges(self, ob):
        """
        Test the specified object to determine if the change set
        will apply without errors
        """
        value = _getValue(ob, self.field)
        if not self.same and value != self.oldValue:
            raise ValueError(
                "Conflict Error during merge", self.field, value, self.oldValue
            )

    def applyChanges(self, ob):
        """Update the specified object with the difference"""
        # Simplistic update
        self.testChanges(ob)
        if not self.same:
            setattr(ob, self.field, self.newValue)

    def ndiff(self):
        """Return a textual diff"""
        r = []
        a = self._parseField(self.oldValue, filename=self.oldFilename)
        b = self._parseField(self.newValue, filename=self.newFilename)
        for tag, alo, ahi, blo, bhi in self.getLineDiffs():
            if tag == "replace":
                plain_replace(a, alo, ahi, b, blo, bhi, r)
            elif tag == "delete":
                dump("-", a, alo, ahi, r)
            elif tag == "insert":
                dump("+", b, blo, bhi, r)
            elif tag == "equal":
                dump(" ", a, alo, ahi, r)
            else:
                raise ValueError("unknown tag %r", tag)
        return "\n".join(r)

    def inline_diff(self):
        css_class = "InlineDiff"
        inlinediff_fmt = self.inlinediff_fmt
        same_fmt = self.same_fmt
        r = []
        a = self._parseField(self.oldValue, filename=self.oldFilename)
        b = self._parseField(self.newValue, filename=self.newFilename)
        for tag, alo, ahi, blo, bhi in self.getLineDiffs():
            if tag == "replace":
                for i in range(alo, ahi):
                    r.append(inlinediff_fmt % (css_class, html_escape(a[i]), ""))
                for i in range(blo, bhi):
                    r.append(inlinediff_fmt % (css_class, "", html_escape(b[i])))
            elif tag == "delete":
                for i in range(alo, ahi):
                    r.append(inlinediff_fmt % (css_class, html_escape(a[i]), ""))
            elif tag == "insert":
                for i in range(blo, bhi):
                    r.append(inlinediff_fmt % (css_class, "", html_escape(b[i])))
            elif tag == "equal":
                for i in range(alo, ahi):
                    r.append(same_fmt % (css_class, html_escape(a[i])))
            else:
                raise ValueError('unknown tag "%s"' % tag)
        return "\n".join(r)


InitializeClass(FieldDiff)


def dump(tag, x, lo, hi, r):
    for i in range(lo, hi):
        r.append(tag + " %s" % x[i])


def plain_replace(a, alo, ahi, b, blo, bhi, r):
    assert alo < ahi and blo < bhi
    # dump the shorter block first -- reduces the burden on short-term
    # memory if the blocks are of very different sizes
    if bhi - blo < ahi - alo:
        dump("+", b, blo, bhi, r)
        dump("-", a, alo, ahi, r)
    else:
        dump("-", a, alo, ahi, r)
        dump("+", b, blo, bhi, r)



from AccessControl.class_init import InitializeClass
from Products.CMFDiffTool.libs import htmldiff
from Products.CMFDiffTool.TextDiff import TextDiff
from Products.CMFDiffTool.utils import html_safe


# Give it a dumb name so it doesn't conflict with all the other html diffs
# around.  This uses Ian Bicking's very nice htmldiff.py from Web Ware for
# Python.
class CMFDTHtmlDiff(TextDiff):
    """Text difference"""

    meta_type = "HTML Diff"

    def inline_diff(self):
        """Return a specialized diff for HTML"""
        a = "\n".join(self._parseField(self.oldValue, filename=self.oldFilename))
        b = "\n".join(self._parseField(self.newValue, filename=self.newFilename))
        return htmldiff.htmldiff(html_safe(a), html_safe(b))

    def _parseField(self, value, filename=None):
        """Use the field's raw value if available."""
        if value is None:
            value = ""
        else:
            value = getattr(value, "raw", value)
        return TextDiff._parseField(self, value, filename)


InitializeClass(CMFDTHtmlDiff)



from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from Products.CMFDiffTool import testing
from Products.CMFDiffTool.choicediff import ChoiceDiff
from Products.CMFDiffTool.choicediff import title_or_value
from Products.CMFDiffTool.interfaces import IDifference
from Products.CMFDiffTool.tests.BaseTestCase import BaseDXTestCase


class ChoiceDiffTestCase(BaseDXTestCase):
    def setUp(self):
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.portal.invokeFactory(
            testing.TEST_CONTENT_TYPE_ID,
            "obj1",
        )
        self.portal.invokeFactory(
            testing.TEST_CONTENT_TYPE_ID,
            "obj2",
        )

        self.obj1 = self.portal["obj1"]
        self.obj2 = self.portal["obj2"]

    def test_should_diff_choice_field(self):
        self._test_diff_choice(
            testing.VOCABULARY_TUPLES[0][0], testing.VOCABULARY_TUPLES[0][0], True
        )
        self._test_diff_choice(
            testing.VOCABULARY_TUPLES[0][0], testing.VOCABULARY_TUPLES[1][0], False
        )
        self._test_diff_choice(
            testing.VOCABULARY_TUPLES[1][0], testing.VOCABULARY_TUPLES[0][0], False
        )
        self._test_diff_choice(
            testing.VOCABULARY_TUPLES[1][0], testing.VOCABULARY_TUPLES[1][0], True
        )

        self._test_diff_choice(testing.VOCABULARY_TUPLES[0][0], None, False)
        self._test_diff_choice(testing.VOCABULARY_TUPLES[1][0], None, False)
        self._test_diff_choice(None, testing.VOCABULARY_TUPLES[0][0], False)
        self._test_diff_choice(None, testing.VOCABULARY_TUPLES[1][0], False)
        self._test_diff_choice(None, None, True)

    def _test_diff_choice(self, value1, value2, same):
        self.obj1.choice = value1
        self.obj2.choice = value2
        diff = ChoiceDiff(self.obj1, self.obj2, "choice")
        self.assertTrue(IDifference.providedBy(diff))
        self.assertEqual(diff.same, same)

        inline_diff = diff.inline_diff()
        if same:
            self.assertFalse(inline_diff)
        else:
            if value1 is not None:
                self.assertTrue(
                    title_or_value(testing.VOCABULARY, value1) in inline_diff
                )
            if value2 is not None:
                self.assertTrue(
                    title_or_value(testing.VOCABULARY, value2) in inline_diff
                )



from AccessControl.class_init import InitializeClass
from plone.namedfile import NamedFile
from Products.CMFDiffTool.BinaryDiff import BinaryDiff
from Products.CMFDiffTool.ListDiff import ListDiff
from Products.CMFDiffTool.TextDiff import TextDiff
from Products.CMFDiffTool.utils import html_escape


FILE_FIELD_TYPES = []

try:
    from plone.namedfile import field

    FILE_FIELD_TYPES.extend([field.NamedFile, field.NamedImage])

    if getattr(field, "HAVE_BLOBS", True):
        FILE_FIELD_TYPES.extend([field.NamedBlobFile, field.NamedBlobImage])
except ImportError:
    pass

FILE_FIELD_TYPES = tuple(FILE_FIELD_TYPES)


def named_file_as_str(f):
    return "" if f is None else "%s (%d bytes)" % (f.filename, len(f.data))


def is_same(old_data, old_filename, new_data, new_filename):
    if old_data != new_data:
        return False

    if (old_filename is not None) and (new_filename is not None):
        return old_filename == new_filename

    return True


class NamedFileBinaryDiff(BinaryDiff):
    def __init__(
        self,
        obj1,
        obj2,
        field,
        id1=None,
        id2=None,
        field_name=None,
        field_label=None,
        schemata=None,
    ):
        self.field = field
        self.label = field_label or field
        self.schemata = schemata or "default"
        self.field_name = field_name or field

        old_field = getattr(obj1, field)
        new_field = getattr(obj2, field)

        self.oldValue = getattr(old_field, "data", None)
        self.newValue = getattr(new_field, "data", None)

        self.id1 = id1 or getattr(obj1, "getId", lambda: None)()
        self.id2 = id2 or getattr(obj2, "getId", lambda: None)()

        self.oldFilename = getattr(old_field, "filename", None)
        self.newFilename = getattr(new_field, "filename", None)

        self.same = is_same(
            self.oldValue, self.oldFilename, self.newValue, self.newFilename
        )

    def _parseField(self, value, filename=None):
        return [
            ""
            if (value is None)
            else named_file_as_str(NamedFile(data=value, filename=filename)),
        ]

    def inline_diff(self):
        css_class = "InlineDiff"
        old = self._parseField(self.oldValue, self.oldFilename)[0]
        new = self._parseField(self.newValue, self.newFilename)[0]

        return (
            ""
            if self.same
            else self.inlinediff_fmt % (css_class, html_escape(old), html_escape(new))
        )


InitializeClass(NamedFileBinaryDiff)


def make_lists_same_length(s1, s2, dummy_element):
    if len(s1) > len(s2):
        s2 += [dummy_element] * (len(s1) - len(s2))
    if len(s2) > len(s1):
        s1 += [dummy_element] * (len(s2) - len(s1))


class NamedFileListDiff(ListDiff):
    """Specialization of `ListDiff` to handle lists of files better."""

    same_fmt = """<div class="%s">%s</div>"""
    inlinediff_fmt = TextDiff.inlinediff_fmt

    def __init__(
        self,
        obj1,
        obj2,
        field,
        id1=None,
        id2=None,
        field_name=None,
        field_label=None,
        schemata=None,
    ):
        ListDiff.__init__(
            self, obj1, obj2, field, id1, id2, field_name, field_label, schemata
        )
        old_values = list(self.oldValue or [])
        new_values = list(self.newValue or [])

        self.same = True
        if len(old_values) != len(new_values):
            self.same = False
        else:
            for old, new in zip(old_values, new_values):
                if not is_same(old.data, old.filename, new.data, new.filename):
                    self.same = False
                    break

    def _parseField(self, value, filename=None):
        value = value or []
        return [named_file_as_str(f) for f in value]

    def inline_diff(self):
        if self.same:
            return None

        css_class = "InlineDiff"

        old_reprs = self._parseField(self.oldValue, None)
        new_reprs = self._parseField(self.newValue, None)

        old_data = [
            {"repr": repr, "data": value.data, "filename": value.filename}
            for (repr, value) in zip(old_reprs, self.oldValue or [])
        ]
        new_data = [
            {"repr": repr, "data": value.data, "filename": value.filename}
            for (repr, value) in zip(new_reprs, self.newValue or [])
        ]

        dummy_dict = {"repr": "", "data": None, "filename": None}
        make_lists_same_length(old_data, new_data, dummy_dict)

        def is_same_dict(d1, d2):
            return is_same(
                d1["data"],
                d1["filename"],
                d2["data"],
                d2["filename"],
            )

        return "\n".join(
            [
                (
                    (self.same_fmt % (css_class, html_escape(d_old["repr"])))
                    if is_same_dict(d_old, d_new)
                    else self.inlinediff_fmt
                    % (
                        css_class,
                        html_escape(d_old["repr"]),
                        html_escape(d_new["repr"]),
                    )
                )
                for (d_old, d_new) in zip(old_data, new_data)
            ]
        )


InitializeClass(NamedFileListDiff)



from plone.namedfile import NamedFile
from Products.CMFDiffTool import namedfile
from Products.CMFDiffTool.interfaces import IDifference

import unittest


class DummyType:
    def __init__(self, files):
        """`files` is a sequence of (data, filename) tuples."""
        self.files = files and [NamedFile(data=d, filename=fn) for (d, fn) in files]


class AsTextDiffTestCase(unittest.TestCase):
    def test_should_diff_file_lists_correctly(self):
        self._test_diff_files(
            [("data1", "filename1")],
            [("data2", "filename2")],
            False,
        )
        self._test_diff_files(
            [("data1", "filename1"), ("datax", "filenamex")],
            [("data1", "filename1"), ("datay", "filenamey")],
            False,
        )
        self._test_diff_files(
            [("data1", "filename1"), ("datax", "filenamex")],
            [("datax", "filenamex"), ("data1", "filename1")],
            False,
        )
        self._test_diff_files(
            [("data1", "filename1")],
            [("data1", "filename1"), ("datax", "filenamex")],
            False,
        )
        self._test_diff_files(
            [("data1", "filename1")],
            [("data1", "filename1")],
            True,
        )
        self._test_diff_files(
            [("data1", "filename1"), ("datax", "filenamex")],
            [("data1", "filename1"), ("datax", "filenamex")],
            True,
        )
        self._test_diff_files(
            [("data1", "filename1"), ("datax", "filenamex")],
            None,
            False,
        )
        self._test_diff_files(
            [("data1", "filename1"), ("datax", "filenamex")],
            [],
            False,
        )
        self._test_diff_files(
            [
                ('<script>alert("Hacker data 1")</script>', "filename1"),
                ('<script>alert("Hacker data 2")</script>', "filename2"),
            ],
            [
                ("data1", '<script>alert("Hacker data")</script>.txt'),
                ('<script>alert("Hacker data 2")</script>', "filename2"),
            ],
            False,
        )
        self._test_diff_files(None, None, True)
        self._test_diff_files([], [], True)
        self._test_diff_files([], None, True)

    def _test_diff_files(self, files1, files2, same):
        diff = namedfile.NamedFileListDiff(
            DummyType(files1), DummyType(files2), "files"
        )
        self.assertTrue(IDifference.providedBy(diff))
        self.assertEqual(diff.same, same)
        inline = diff.inline_diff()
        self.assertNotEqual(bool(inline), same)
        if inline:
            # No hacker can catch us unawares.
            self.assertNotIn("<script", inline)



"""
htmldiff.py
(C) Ian Bicking <ianb@colorstudy.com>

Finds the differences between two HTML files.  *Not* line-by-line
comparison (more word-by-word).

Command-line usage:
  ./htmldiff.py test1.html test2.html

Better results if you use mxTidy first.  The output is HTML.
"""
from difflib import SequenceMatcher
from html import escape
from io import StringIO

import re


def htmlEncode(s, esc=escape):
    return esc(s, 1)


commentRE = re.compile(r"<!--.*?-->", re.S)
tagRE = re.compile(r"<.*?>", re.S)
headRE = re.compile(r"<\s*head\s*>", re.S | re.I)


class HTMLMatcher(SequenceMatcher):
    def __init__(self, source1, source2):
        SequenceMatcher.__init__(self, None, source1, source2)

    def set_seq1(self, a):
        SequenceMatcher.set_seq1(self, self.splitHTML(a))

    def set_seq2(self, b):
        SequenceMatcher.set_seq2(self, self.splitHTML(b))

    def splitTags(self, t):
        result = []
        pos = 0
        while 1:
            match = tagRE.search(t, pos=pos)
            if not match:
                result.append(t[pos:])
                break
            result.append(t[pos : match.start()])
            result.append(match.group(0))
            pos = match.end()
        return result

    def splitWords(self, t):
        return t.strip().split()

    def splitHTML(self, t):
        t = commentRE.sub("", t)
        r = self.splitTags(t)
        result = []
        for item in r:
            if item.startswith("<"):
                result.append(item)
            else:
                result.extend(self.splitWords(item))
        return result

    def htmlDiff(self, addStylesheet=False):
        opcodes = self.get_opcodes()
        a = self.a
        b = self.b
        out = StringIO()
        #  print [o[0] for o in opcodes]
        for tag, i1, i2, j1, j2 in opcodes:
            if tag == "equal":
                for item in a[i1:i2]:
                    out.write(item)
                    out.write(" ")
            if tag == "delete" or tag == "replace":
                self.textDelete(a[i1:i2], out)
            if tag == "insert" or tag == "replace":
                self.textInsert(b[j1:j2], out)
        html = out.getvalue()
        out.close()
        if addStylesheet:
            html = self.addStylesheet(html, self.stylesheet())
        return html

    def textDelete(self, lst, out):
        inSpan = False
        for item in lst:
            if item.startswith("<"):
                if inSpan:
                    out.write(self.endDeleteText())
                    inSpan = False
                out.write(self.formatDeleteTag(item))
            else:
                if not inSpan:
                    out.write(self.startDeleteText())
                    inSpan = True
                out.write(item)
                out.write(" ")
        if inSpan:
            out.write(self.endDeleteText())

    def textInsert(self, lst, out):
        inSpan = False
        for item in lst:
            if item.startswith("<"):
                if inSpan:
                    out.write(self.endInsertText())
                    inSpan = False
                out.write(self.formatInsertTag(item))
                out.write(item)
                out.write(" ")
            else:
                if not inSpan:
                    out.write(self.startInsertText())
                    inSpan = True
                out.write(item)
                out.write(" ")
        if inSpan:
            out.write(self.endInsertText())

    def stylesheet(self):
        return """
.insert { background-color: #aaffaa }
.delete { background-color: #ff8888 }
.tagInsert { background-color: #007700; color: #ffffff }
.tagDelete { background-color: #770000; color: #ffffff }
"""

    def addStylesheet(self, html, ss):
        match = headRE.search(html)
        if match:
            pos = match.end()
        else:
            pos = 0
        return '{}<style type="text/css"><!--\n{}\n--></style>{}'.format(
            html[:pos], ss, html[pos:]
        )

    def startInsertText(self):
        return '<span class="insert">'

    def endInsertText(self):
        return "</span> "

    def startDeleteText(self):
        return '<span class="delete">'

    def endDeleteText(self):
        return "</span> "

    def formatInsertTag(self, tag):
        return '<span class="tagInsert">insert: <tt>%s</tt></span> ' % htmlEncode(tag)

    def formatDeleteTag(self, tag):
        return '<span class="tagDelete">delete: <tt>%s</tt></span> ' % htmlEncode(tag)


class NoTagHTMLMatcher(HTMLMatcher):
    def formatInsertTag(self, tag):
        return ""

    def formatDeleteTag(self, tag):
        return ""


def htmldiff(source1, source2, addStylesheet=False):
    """
    Return the difference between two pieces of HTML

        >>> htmldiff('test1', 'test2')
        '<span class="delete">test1 </span> <span class="insert">test2 </span> '  # NOQA
        >>> htmldiff('test1', 'test1')
        'test1 '
        >>> htmldiff('<b>test1</b>', '<i>test1</i>')
        '<span class="tagDelete">delete: <tt>&lt;b&gt;</tt></span> <span class="tagInsert">insert: <tt>&lt;i&gt;</tt></span> <i> test1 <span class="tagDelete">delete: <tt>&lt;/b&gt;</tt></span> <span class="tagInsert">insert: <tt>&lt;/i&gt;</tt></span> </i> '
    """
    h = HTMLMatcher(source1, source2)
    return h.htmlDiff(addStylesheet)


def diffFiles(f1, f2):
    source1 = open(f1).read()
    source2 = open(f2).read()
    return htmldiff(source1, source2, True)


class SimpleHTMLMatcher(HTMLMatcher):
    """
    Like HTMLMatcher, but returns a simpler diff
    """

    def startInsertText(self):
        return "+["

    def endInsertText(self):
        return "]"

    def startDeleteText(self):
        return "-["

    def endDeleteText(self):
        return "]"

    def formatInsertTag(self, tag):
        return "+[%s]" % tag

    def formatDeleteTag(self, tag):
        return "-[%s]" % tag


def simplehtmldiff(source1, source2):
    """
    Simpler form of htmldiff; mostly for testing, like:

        >>> simplehtmldiff('test1', 'test2')
        '-[test1 ]+[test2 ]'
        >>> simplehtmldiff('<b>Hello world!</b>', '<i>Hello you!</i>')
        '-[<b>]+[<i>]<i> Hello -[world! ]-[</b>]+[you! ]+[</i>]</i> '
    """
    h = SimpleHTMLMatcher(source1, source2)
    return h.htmlDiff()


class TextMatcher(HTMLMatcher):
    def set_seq1(self, a):
        SequenceMatcher.set_seq1(self, a.split("\n"))

    def set_seq2(self, b):
        SequenceMatcher.set_seq2(self, b.split("\n"))

    def htmlDiff(self, addStylesheet=False):
        opcodes = self.get_opcodes()
        a = self.a
        b = self.b
        out = StringIO()
        for tag, i1, i2, j1, j2 in opcodes:
            if tag == "equal":
                self.writeLines(a[i1:i2], out)
            if tag == "delete" or tag == "replace":
                out.write(self.startDeleteText())
                self.writeLines(a[i1:i2], out)
                out.write(self.endDeleteText())
            if tag == "insert" or tag == "replace":
                out.write(self.startInsertText())
                self.writeLines(b[j1:j2], out)
                out.write(self.endInsertText())
        html = out.getvalue()
        out.close()
        if addStylesheet:
            html = self.addStylesheet(html, self.stylesheet())
        return html

    def writeLines(self, lines, out):
        for line in lines:
            line = htmlEncode(line)
            line = line.replace("  ", "&nbsp; ")
            line = line.replace("\t", "&nbsp; &nbsp; &nbsp; &nbsp; ")
            if line.startswith(" "):
                line = "&nbsp;" + line[1:]
            out.write("<tt>%s</tt><br>\n" % line)


if __name__ == "__main__":
    import sys

    if not sys.argv[1:]:
        print("Usage: %s file1 file2" % sys.argv[0])  # NOQA
        print("or to test: %s test" % sys.argv[0])  # NOQA
    elif sys.argv[1] == "test" and not sys.argv[2:]:
        import doctest

        doctest.testmod()
    else:
        print(diffFiles(sys.argv[1], sys.argv[2]))  # NOQA



from Products.CMFCore.utils import getToolByName
from Products.CMFDiffTool.dexteritydiff import DexterityCompoundDiff
from Products.CMFDiffTool.tests.BaseTestCase import BaseDXTestCase


class InstallTestCase(BaseDXTestCase):
    def test_compound_diff_type_should_be_registered(self):
        diff_tool = getToolByName(self.portal, "portal_diff")
        self.assertTrue(DexterityCompoundDiff.meta_type in diff_tool.listDiffTypes())
        self.assertTrue(diff_tool.getDiffType(DexterityCompoundDiff.meta_type))



"""Initialize CMFDiffTool Product"""
# Set up a MessageFactory for the cmfdifftool domain
from zope.i18nmessageid import MessageFactory


CMFDiffToolMessageFactory = MessageFactory("plone")

from Products.CMFCore.utils import ToolInit  # NOQA
from Products.CMFDiffTool import BinaryDiff  # NOQA
from Products.CMFDiffTool import CMFDiffTool  # NOQA
from Products.CMFDiffTool import CMFDTHtmlDiff  # NOQA
from Products.CMFDiffTool import FieldDiff  # NOQA
from Products.CMFDiffTool import ListDiff  # NOQA
from Products.CMFDiffTool import TextDiff  # NOQA


CMFDiffTool.registerDiffType(BinaryDiff.BinaryDiff)
CMFDiffTool.registerDiffType(FieldDiff.FieldDiff)
CMFDiffTool.registerDiffType(ListDiff.ListDiff)
CMFDiffTool.registerDiffType(TextDiff.TextDiff)
CMFDiffTool.registerDiffType(TextDiff.AsTextDiff)
CMFDiffTool.registerDiffType(CMFDTHtmlDiff.CMFDTHtmlDiff)

# Soft plone.namedfile dependency
try:
    from Products.CMFDiffTool import namedfile
except ImportError:
    pass
else:
    CMFDiffTool.registerDiffType(namedfile.NamedFileBinaryDiff)
    CMFDiffTool.registerDiffType(namedfile.NamedFileListDiff)

# Soft Dexterity dependency
try:
    from Products.CMFDiffTool import choicediff
    from Products.CMFDiffTool import dexteritydiff
except ImportError:
    pass
else:
    CMFDiffTool.registerDiffType(choicediff.ChoiceDiff)
    CMFDiffTool.registerDiffType(dexteritydiff.DexterityCompoundDiff)

tools = (CMFDiffTool.CMFDiffTool,)


def initialize(context):
    ToolInit(
        "CMF Diff Tool",
        tools=tools,
        icon="tool.gif",
    ).initialize(context)



#
# CMFDiffTool tests
#
from Acquisition import aq_base
from os import linesep
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FUNCTIONAL_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.base.utils import safe_hasattr
from Products.CMFCore.utils import getToolByName
from Products.CMFDiffTool.ChangeSet import BaseChangeSet
from Products.CMFDiffTool.dexteritydiff import DexterityCompoundDiff
from unittest import TestCase


class TestChangeSet(TestCase):
    """Tests for ChangeSet objects"""

    layer = PLONE_APP_CONTENTTYPES_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.folder = self.portal
        self.p_diff = getToolByName(self.portal, "portal_diff")
        cs = BaseChangeSet("my_changeset")
        # ChangeSet needs an acquisition wrapper
        self.cs = cs.__of__(self.portal)
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])

    def testInterface(self):
        """Ensure that tool instances implement the portal_diff interface"""
        from Products.CMFDiffTool.interfaces import IChangeSet

        self.assertTrue(IChangeSet.implementedBy(BaseChangeSet))

    def setupTestObjects(self):
        self.folder.invokeFactory("Document", "doc1", title="My Title")
        self.folder.manage_pasteObjects(self.folder.manage_copyObjects(["doc1"]))
        cdd = DexterityCompoundDiff(self.folder["doc1"], self.folder["doc1"], "")
        self.len_diff = len(cdd._diffs)

    def setupTestFolders(self):
        self.folder.invokeFactory("Folder", "folder1", title="My Folder Title")
        self.folder.folder1.invokeFactory("Document", "doc1", title="My Title1")
        self.folder.folder1.invokeFactory("Document", "doc2", title="My Title2")
        self.folder.folder1.invokeFactory("Document", "doc3", title="My Title3")
        self.folder.manage_pasteObjects(self.folder.manage_copyObjects(["folder1"]))
        cdd = DexterityCompoundDiff(
            self.folder["folder1"]["doc1"], self.folder["folder1"]["doc1"], ""
        )
        self.len_diff = len(cdd._diffs)

    def testChangeSetUnchanged(self):
        self.setupTestObjects()
        self.cs.computeDiff(self.folder["doc1"], self.folder["copy_of_doc1"])
        diffs = self.cs.getDiffs()
        self.assertEqual(len(diffs), self.len_diff)
        self.assertTrue(diffs[0].same)

    def testChangeSetChanged(self):
        self.setupTestObjects()
        self.folder.copy_of_doc1.setTitle("My New Title")
        self.cs.computeDiff(self.folder.doc1, self.folder.copy_of_doc1)
        diffs = self.cs.getDiffs()
        self.assertEqual(len(diffs), self.len_diff)
        self.assertFalse(diffs[0].same)
        self.assertEqual(diffs[0].ndiff(), "- My Title%s+ My New Title" % linesep)

    def testChangeSetFolderUnchanged(self):
        self.setupTestFolders()
        self.cs.computeDiff(self.folder.folder1, self.folder.copy_of_folder1)
        diffs = self.cs.getDiffs()
        self.assertEqual(len(diffs), 14)
        self.assertTrue(diffs[0].same)
        sub_cs = self.cs.getSubDiffs()
        self.assertEqual(len(sub_cs), 3)
        for i in range(len(sub_cs)):
            self.assertTrue(isinstance(sub_cs[i], BaseChangeSet))
            sub_diffs = sub_cs[i].getDiffs()
            self.assertEqual(len(sub_diffs), self.len_diff)
            self.assertTrue(sub_cs[0].same)
            self.assertTrue(sub_cs[1].same)
            self.assertTrue(sub_cs[2].same)

    def testChangeSetFolderChanged(self):
        self.setupTestFolders()
        self.folder.copy_of_folder1.setTitle("My New Title")
        self.cs.computeDiff(self.folder.folder1, self.folder.copy_of_folder1)
        diffs = self.cs.getDiffs()
        self.assertEqual(len(diffs), 14)
        self.assertFalse(diffs[0].same)
        self.assertEqual(
            diffs[0].ndiff(), "- My Folder Title%s+ My New Title" % linesep
        )
        self.assertFalse(self.cs._added)
        self.assertFalse(self.cs._removed)
        sub_cs = self.cs.getSubDiffs()
        self.assertEqual(len(sub_cs), 3)
        # The sub diffs should show no changes
        for i in range(len(sub_cs)):
            self.assertTrue(isinstance(sub_cs[i], BaseChangeSet))
            sub_diffs = sub_cs[i].getDiffs()
            self.assertEqual(len(sub_diffs), self.len_diff)
            self.assertTrue(sub_diffs[0].same)

    def testChangeSetFolderDocChanged(self):
        self.setupTestFolders()
        self.folder.copy_of_folder1.doc1.setTitle("My New Title")
        self.cs.computeDiff(self.folder.folder1, self.folder.copy_of_folder1)
        diffs = self.cs.getDiffs()
        self.assertEqual(len(diffs), 14)
        self.assertTrue(diffs[0].same)
        self.assertTrue(diffs[1].same)
        self.assertFalse(self.cs._added)
        self.assertFalse(self.cs._removed)
        sub_cs = self.cs.getSubDiffs()
        self.assertEqual(len(sub_cs), 3)
        for i in range(len(sub_cs)):
            self.assertTrue(isinstance(sub_cs[i], BaseChangeSet))
            sub_diffs = sub_cs[i].getDiffs()
            self.assertEqual(len(sub_diffs), self.len_diff)
            # doc1 has changed
            if sub_cs[i].getId() == "doc1":
                self.assertFalse(sub_diffs[0].same)
                self.assertEqual(
                    sub_diffs[0].ndiff(), "- My Title1%s+ My New Title" % linesep
                )
            else:
                self.assertTrue(sub_diffs[0].same)

    def testChangeSetFolderDocRemoved(self):
        self.setupTestFolders()
        self.folder.copy_of_folder1.manage_delObjects("doc1")
        self.cs.computeDiff(self.folder.folder1, self.folder.copy_of_folder1)
        diffs = self.cs.getDiffs()
        self.assertEqual(len(diffs), 14)
        self.assertTrue(diffs[0].same)
        self.assertTrue(diffs[1].same)
        sub_cs = self.cs.getSubDiffs()
        # We only have two potentially changed objects
        self.assertEqual(len(sub_cs), 2)
        # The sub diffs should show no changes
        for i in range(len(sub_cs)):
            self.assertTrue(isinstance(sub_cs[i], BaseChangeSet))
            sub_diffs = sub_cs[i].getDiffs()
            self.assertEqual(len(sub_diffs), self.len_diff)
            self.assertTrue(sub_diffs[0].same)
        self.assertFalse(self.cs._added)
        self.assertEqual(list(self.cs._removed), ["doc1"])

    def testChangeSetFolderDocAdded(self):
        self.setupTestFolders()
        self.folder.copy_of_folder1.invokeFactory(
            "Document", "doc4", title="My Doc Title"
        )
        self.cs.computeDiff(self.folder.folder1, self.folder.copy_of_folder1)
        diffs = self.cs.getDiffs()
        self.assertEqual(len(diffs), 14)
        self.assertTrue(diffs[0].same)
        self.assertTrue(diffs[1].same)
        sub_cs = self.cs.getSubDiffs()
        self.assertEqual(len(sub_cs), 3)
        # The sub diffs should show no changes
        for i in range(len(sub_cs)):
            self.assertTrue(isinstance(sub_cs[i], BaseChangeSet))
            sub_diffs = sub_cs[i].getDiffs()
            self.assertEqual(len(sub_diffs), self.len_diff)
            self.assertTrue(sub_diffs[0].same)
        self.assertFalse(self.cs._removed)
        self.assertEqual(list(self.cs._added), ["doc4"])

    def testChangeSetFolderReordered(self):
        self.setupTestFolders()
        if safe_hasattr(aq_base(self.folder.copy_of_folder1), "moveObjectsToTop"):
            self.folder.copy_of_folder1.moveObjectsToTop(["doc3"])
        elif safe_hasattr(aq_base(self.folder.copy_of_folder1), "moveObjectsByDelta"):
            self.folder.copy_of_folder1.moveObjectsByDelta(["doc3"], -3)
        else:
            # We don't have an orderable folder give up
            return
        self.cs.computeDiff(self.folder.folder1, self.folder.copy_of_folder1)
        diffs = self.cs.getDiffs()
        self.assertEqual(len(diffs), 14)
        self.assertTrue(diffs[0].same)
        self.assertFalse(self.cs._added)
        self.assertFalse(self.cs._removed)
        sub_cs = self.cs.getSubDiffs()
        self.assertEqual(len(sub_cs), 3)
        # The sub diffs should show no changes
        for i in range(len(sub_cs)):
            self.assertTrue(isinstance(sub_cs[i], BaseChangeSet))
            sub_diffs = sub_cs[i].getDiffs()
            self.assertEqual(len(sub_diffs), self.len_diff)
            self.assertTrue(sub_diffs[0].same)
        # XXX we need an explicit way of noting reorders

    def testChangeSetFolderComplex(self):
        self.setupTestFolders()
        # Add a new sub object
        self.folder.copy_of_folder1.invokeFactory(
            "Document", "doc4", title="My Doc Title"
        )
        # Delete a sub object
        self.folder.copy_of_folder1.manage_delObjects("doc2")
        # Change one object
        self.folder.copy_of_folder1.doc3.setTitle("My New Title")
        # Change the folder itself
        self.folder.copy_of_folder1.setTitle("My New Title")
        # Move the changed object
        if safe_hasattr(aq_base(self.folder.copy_of_folder1), "moveObjectsToTop"):
            self.folder.copy_of_folder1.moveObjectsToTop(["doc3"])
        elif safe_hasattr(aq_base(self.folder.copy_of_folder1), "moveObjectsByDelta"):
            self.folder.copy_of_folder1.moveObjectsByDelta(["doc3"], -3)
        else:
            # We don't have an orderable folder give up
            return

        self.cs.computeDiff(self.folder["folder1"], self.folder["copy_of_folder1"])
        diffs = self.cs.getDiffs()
        self.assertEqual(len(diffs), 14)
        self.assertFalse(diffs[0].same)
        self.assertEqual(
            diffs[0].ndiff(), "- My Folder Title%s+ My New Title" % linesep
        )
        self.assertEqual(list(self.cs._added), ["doc4"])
        self.assertEqual(list(self.cs._removed), ["doc2"])
        sub_cs = self.cs.getSubDiffs()
        # We only have two potentially changed objects
        self.assertEqual(len(sub_cs), 2)
        # The sub diffs should show no changes
        for i in range(len(sub_cs)):
            self.assertTrue(isinstance(sub_cs[i], BaseChangeSet))
            sub_diffs = sub_cs[i].getDiffs()
            self.assertEqual(len(sub_diffs), self.len_diff)
            if sub_cs[i].getId() == "doc3":
                self.assertFalse(sub_diffs[0].same)
                self.assertEqual(
                    sub_diffs[0].ndiff(), "- My Title3%s+ My New Title" % linesep
                )
            else:
                self.assertTrue(sub_diffs[1].same)
        # XXX we need an explicit way of noting reorders



#
# CMFDiffTool tests
#

from plone.app.testing import PLONE_INTEGRATION_TESTING
from Products.CMFCore.utils import getToolByName
from Products.CMFDiffTool.CMFDiffTool import registerDiffType
from Products.CMFDiffTool.CMFDiffTool import unregisterDiffType
from unittest import TestCase
from zExceptions import BadRequest


class DummyDiff:
    meta_type = "Dummy Diff Type"


class DummyDiff2:
    meta_type = "Second Dummy Diff Type"


class TestDiffTool(TestCase):
    """Test the portal_diff tool"""

    layer = PLONE_INTEGRATION_TESTING

    def setUp(self):
        self.p_diff = getToolByName(self.layer["portal"], "portal_diff")
        # clear pt_diff registry
        self.p_diff._pt_diffs = {}

        # patch portal_types to list `Document` in the listContentTypes
        # a plausibility check is done in the `setDiffForPortalType` method
        # but we have no content registry
        portal_types = getToolByName(self.layer["portal"], "portal_types")
        self._old_listContentTypes = portal_types.listContentTypes
        portal_types.listContentTypes = lambda: ["Document"]
        registerDiffType(DummyDiff)

    def tearDown(self):
        portal_types = getToolByName(self.layer["portal"], "portal_types")
        portal_types.listContentTypes = self._old_listContentTypes

    def testInterface(self):
        """Ensure that tool instances implement the portal_diff interface"""
        from Products.CMFDiffTool.interfaces.portal_diff import portal_diff

        self.assertTrue(portal_diff.providedBy(self.p_diff))

    def testRegisterDiffType(self):
        """Test registration of Diff types"""
        unregisterDiffType(DummyDiff)
        self.assertNotIn("Dummy Diff Type", self.p_diff.listDiffTypes())
        registerDiffType(DummyDiff)
        self.assertIn("Dummy Diff Type", self.p_diff.listDiffTypes())

    def testSetDiff(self):
        """Test setDiffForPortalType() method"""
        d = {"field1": "TestDiff", "field2": "Dummy Diff Type"}
        self.p_diff.setDiffForPortalType("Document", d)
        self.assertEqual(self.p_diff.getDiffForPortalType("Document"), d)

    def testSetDiffReplaces(self):
        """Test that setDiffForPortalType() replaces old data"""
        d1 = {"field1": "TestDiff", "field2": "Dummy Diff Type"}
        d2 = {"field3": "Dummy Diff Type"}
        self.p_diff.setDiffForPortalType("Document", d1)
        self.p_diff.setDiffForPortalType("Document", d2)
        self.assertEqual(self.p_diff.getDiffForPortalType("Document"), d2)

    def testSingleSetDiffField(self):
        """Test setDiffField method"""
        self.p_diff.setDiffField("Document", "title", "Dummy Diff Type")
        self.assertEqual(
            self.p_diff.getDiffForPortalType("Document"), {"title": "Dummy Diff Type"}
        )

    def testMultipleSetDiffField(self):
        """
        Test setDiffField method adding a second field to one content type
        """
        self.p_diff.setDiffField("Document", "title", "Dummy Diff Type")
        self.p_diff.setDiffField("Document", "description", "Dummy Diff Type")
        d = {"title": "Dummy Diff Type", "description": "Dummy Diff Type"}
        self.assertEqual(self.p_diff.getDiffForPortalType("Document"), d)

    def testReplaceSetDiffField(self):
        """Test that setDiffField does a replace for existing fields"""
        registerDiffType(DummyDiff2)
        self.p_diff.setDiffField("Document", "title", "Dummy Diff Type")
        self.p_diff.setDiffField("Document", "title", "Second Dummy Diff Type")
        d = {"title": "Second Dummy Diff Type"}
        self.assertEqual(self.p_diff.getDiffForPortalType("Document"), d)
        unregisterDiffType(DummyDiff2)

    def testSetDiffFieldNameFailure(self):
        self.assertRaises(
            BadRequest, self.p_diff.setDiffField, "Bob", "title", "Dummy Diff Type"
        )

    def testSetDiffFieldBlankFieldFailure(self):
        self.assertRaises(
            BadRequest, self.p_diff.setDiffField, "Document", "", "Dummy Diff Type"
        )

    def testSetDiffFieldInvalidDiffFailure(self):
        self.assertRaises(
            BadRequest, self.p_diff.setDiffField, "Document", "title", "NoDiff"
        )

    def beforeTearDown(self):
        # Undo changes that don't get rolled back (i.e. module level changes)
        unregisterDiffType(DummyDiff)



from os import linesep
from plone.app.testing import PLONE_INTEGRATION_TESTING
from Products.CMFDiffTool.TextDiff import TextDiff
from unittest import TestCase


_marker = []


class A:
    attribute = "कामसूत्र"

    def method(self):
        return "method कामसूत्र"


class B:
    attribute = "過労死"

    def method(self):
        return "method 過労死"


class H:
    attribute = '<script>alert("Hacker value")</script>'

    def method(self):
        return '<script>alert("Hacker method value")</script>'


class TestTextDiff(TestCase):
    """Test the TextDiff class"""

    layer = PLONE_INTEGRATION_TESTING

    def testInterface(self):
        """Ensure that tool instances implement the portal_diff interface"""
        from Products.CMFDiffTool.interfaces import IDifference

        self.assertTrue(IDifference.implementedBy(TextDiff))

    def testAttributeSame(self):
        """Test attribute with same value"""
        a = A()
        fd = TextDiff(a, a, "attribute")
        self.assertTrue(fd.same)

    def testMethodSame(self):
        """Test method with same value"""
        a = A()
        fd = TextDiff(a, a, "method")
        self.assertTrue(fd.same)

    def testAttributeDiff(self):
        """Test attribute with different value"""
        a = A()
        b = B()
        fd = TextDiff(a, b, "attribute")
        self.assertFalse(fd.same)

    def testMethodDiff(self):
        """Test method with different value"""
        a = A()
        b = B()
        fd = TextDiff(a, b, "method")
        self.assertFalse(fd.same)

    def testGetLineDiffsSame(self):
        """test getLineDiffs() method with same value"""
        a = A()
        fd = TextDiff(a, a, "attribute")
        expected = [("equal", 0, 1, 0, 1)]
        self.assertEqual(fd.getLineDiffs(), expected)

    def testGetLineDiffsDifferent(self):
        """test getLineDiffs() method with different value"""
        a = A()
        b = B()
        fd = TextDiff(a, b, "attribute")
        expected = [("replace", 0, 1, 0, 1)]
        self.assertEqual(fd.getLineDiffs(), expected)

    def testSameText(self):
        """Test text diff output with same value"""
        a = A()
        fd = TextDiff(a, a, "attribute")
        self.assertEqual(fd.ndiff(), "  कामसूत्र")

    def testDiffText(self):
        """Test text diff output with different value"""
        a = A()
        b = B()
        expected = "- कामसूत्र%s+ 過労死" % linesep
        fd = TextDiff(a, b, "attribute")
        self.assertEqual(fd.ndiff(), expected)

    def testUnifiedDiff(self):
        """Test text diff output with different value"""
        a = A()
        b = B()

        expected = """--- version1

+++ version2

@@ -1 +1 @@

-कामसूत्र
+過労死"""
        fd = TextDiff(a, b, "attribute", "version1", "version2")
        self.assertEqual(fd.unified_diff(), expected)

    def testHTMLDiff(self):
        """Test text diff output with different value"""
        a = A()
        b = B()
        h = H()
        expected = """
    <table class="diff" id="difflib_chg_to0__top"
           cellspacing="0" cellpadding="0" rules="groups" >
        <colgroup></colgroup> <colgroup></colgroup> <colgroup></colgroup>
        <colgroup></colgroup> <colgroup></colgroup> <colgroup></colgroup>
        <thead><tr><th class="diff_next"><br /></th><th colspan="2" class="diff_header">None</th><th class="diff_next"><br /></th><th colspan="2" class="diff_header">None</th></tr></thead>
        <tbody>
            <tr><td class="diff_next" id="difflib_chg_to0__0"><a href="#difflib_chg_to0__top">t</a></td><td class="diff_header" id="from0_1">1</td><td nowrap="nowrap"><span class="diff_sub">कामसूत्र</span></td><td class="diff_next"><a href="#difflib_chg_to0__top">t</a></td><td class="diff_header" id="to0_1">1</td><td nowrap="nowrap"><span class="diff_add">過労死</span></td></tr>
        </tbody>
    </table>"""  # NOQA
        fd = TextDiff(a, b, "attribute")
        self.assertEqual(fd.html_diff(), expected)

        fd = TextDiff(a, h, "attribute")
        # h.attribute contains a script, and this should be escaped.
        self.assertNotIn(h.attribute, fd.html_diff())
        self.assertIn("&gt;", fd.html_diff())

    def testInlineDiff(self):
        """Test text inline diff output with different value"""
        a = A()
        b = B()
        h = H()
        fd = TextDiff(a, b, "attribute")
        self.assertIn('class="InlineDiff FilenameDiff"', fd.inline_diff())

        fd = TextDiff(a, h, "attribute")
        self.assertIn('class="InlineDiff FilenameDiff"', fd.inline_diff())
        # h.attribute contains a script, and this should be escaped.
        self.assertNotIn(h.attribute, fd.inline_diff())
        self.assertIn("&gt;", fd.inline_diff())



from setuptools import find_packages
from setuptools import setup


version = "4.0.1"

setup(
    name="Products.CMFDiffTool",
    version=version,
    description="Diff tool for Plone",
    long_description=(open("README.rst").read() + "\n" + open("CHANGES.rst").read()),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Framework :: Plone",
        "Framework :: Plone :: 6.0",
        "Framework :: Plone :: Core",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Topic :: Internet :: WWW/HTTP",
    ],
    keywords="Diff Plone",
    author="Brent Hendricks",
    author_email="plone-developers@lists.sourceforge.net",
    url="https://github.com/plone/Products.CMFDiffTool",
    license="GPL",
    packages=find_packages(),
    namespace_packages=["Products"],
    include_package_data=True,
    zip_safe=False,
    extras_require=dict(
        test=[
            "zope.component",
            "plone.app.testing",
            "plone.namedfile",
            "plone.app.contenttypes[test]",
            "zope.intid",
        ]
    ),
    python_requires=">=3.8",
    install_requires=[
        "ExtensionClass",
        "Products.GenericSetup",
        "Products.PortalTransforms",
        "Zope",
        "setuptools",
        "plone.base",
        "z3c.relationfield",
    ],
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,
)



#
# CMFDiffTool tests
#
from os import linesep
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from Products.CMFDiffTool import testing
from Products.CMFDiffTool.interfaces import IDifference
from Products.CMFDiffTool.ListDiff import ListDiff
from Products.CMFDiffTool.tests.BaseTestCase import BaseDXTestCase


_marker = []


class A:
    attribute = [1, 2, 3]


class B:
    attribute = [1, 2, 3, 4]


class HList:
    attribute = ['<script>alert("Hacker value")</script>']


class C:
    attribute = {"a": 1, "b": 2}


class D:
    attribute = {"a": 1, "b": 2, "c": 3}


class HDict:
    attribute = {"a": '<script>alert("Hacker value")</script>'}


class TestListDiff(BaseDXTestCase):
    """Test the ListDiff class"""

    def setUp(self):
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.portal.invokeFactory(
            testing.TEST_CONTENT_TYPE_ID,
            "obj1",
        )
        self.portal.invokeFactory(
            testing.TEST_CONTENT_TYPE_ID,
            "obj2",
        )

        self.obj1 = self.portal["obj1"]
        self.obj2 = self.portal["obj2"]

    def testInterface(self):
        """Ensure that tool instances implement the portal_diff interface"""
        self.assertTrue(IDifference.implementedBy(ListDiff))

    def testInvalidValue(self):
        """Test if no error with invalid values"""
        a = A()
        a.attribute = []
        b = A()

        b.attribute = None
        diff = ListDiff(a, b, "attribute")
        self.assertEqual([("insert", 0, 0, 0, 1)], diff.getLineDiffs())

        b.attribute = 0
        diff = ListDiff(a, b, "attribute")
        self.assertEqual([("insert", 0, 0, 0, 1)], diff.getLineDiffs())

        b.attribute = ""
        diff = ListDiff(a, b, "attribute")
        self.assertEqual([("insert", 0, 0, 0, 1)], diff.getLineDiffs())

    def testAttributeSame(self):
        """Test attribute with same value"""
        a = A()
        diff = ListDiff(a, a, "attribute")
        self.assertTrue(diff.same)

    def testAttributeDiff(self):
        """Test attribute with different value"""
        a = A()
        b = B()
        diff = ListDiff(a, b, "attribute")
        self.assertFalse(diff.same)

    def testGetLineDiffsSame(self):
        """test getLineDiffs() method with same value"""
        a = A()
        diff = ListDiff(a, a, "attribute")
        expected = [("equal", 0, 3, 0, 3)]
        self.assertEqual(diff.getLineDiffs(), expected)

    def testGetLineDiffsDifferent(self):
        """test getLineDiffs() method with different value"""
        a = A()
        b = B()
        diff = ListDiff(a, b, "attribute")
        expected = [("equal", 0, 3, 0, 3), ("insert", 3, 3, 3, 4)]
        self.assertEqual(diff.getLineDiffs(), expected)

    def testSameText(self):
        """Test text diff output with no diff"""
        a = A()
        diff = ListDiff(a, a, "attribute")
        expected = "  1{linesep}  2{linesep}  3".format(linesep=linesep)
        self.assertEqual(diff.ndiff(), expected)

    def testDiffText(self):
        """Test text diff output with no diff"""
        a = A()
        b = B()
        expected = "  1{linesep}  2{linesep}  3{linesep}+ 4".format(linesep=linesep)
        diff = ListDiff(a, b, "attribute")
        self.assertEqual(diff.ndiff(), expected)

    def test_inline_diff(self):
        a = A()
        b = B()
        expected = """<div class="InlineDiff">1</div>
<div class="InlineDiff">2</div>
<div class="InlineDiff">3</div>
<div class="InlineDiff">
    <div class="diff_sub"></div>
    <div class="diff_add">4</div>
</div>"""
        diff = ListDiff(a, b, "attribute")
        self.assertEqual(diff.inline_diff(), expected)

    def test_inline_diff_hacker_list(self):
        a = A()
        h = HList()
        diff = ListDiff(a, h, "attribute")
        # The script tag should be escaped.
        self.assertNotIn("<script", diff.inline_diff())
        self.assertIn("&gt;", diff.inline_diff())

    def test_inline_diff_hacker_dict(self):
        d = D()
        h = HDict()
        diff = ListDiff(d, h, "attribute")
        # The script tag should be escaped.
        self.assertNotIn("<script", diff.inline_diff())
        self.assertIn("&gt;", diff.inline_diff())

    def test_inline_diff_vocabulary(self):
        # unchanged, with vocabulary title
        expected = '<div class="InlineDiff">First Title</div>'
        self._test_diff_list(
            [testing.VOCABULARY_TUPLES[0][0]],
            [testing.VOCABULARY_TUPLES[0][0]],
            True,
            expected,
        )
        # unchanged, without vocabulary title
        expected = '<div class="InlineDiff">second_value</div>'
        self._test_diff_list(
            [testing.VOCABULARY_TUPLES[1][0]],
            [testing.VOCABULARY_TUPLES[1][0]],
            True,
            expected,
        )
        # changed: add value, with vocabulary title
        expected = """<div class="InlineDiff">
    <div class="diff_sub"></div>
    <div class="diff_add">First Title</div>
</div>"""
        self._test_diff_list([], [testing.VOCABULARY_TUPLES[0][0]], False, expected)
        # changed: replaced unique value by another one, displaying titles
        expected = """<div class="InlineDiff">
    <div class="diff_sub">First Title</div>
    <div class="diff_add"></div>
</div>
<div class="InlineDiff">
    <div class="diff_sub"></div>
    <div class="diff_add">Third Title</div>
</div>"""
        self._test_diff_list(
            [testing.VOCABULARY_TUPLES[0][0]],
            [testing.VOCABULARY_TUPLES[2][0]],
            False,
            expected,
        )
        # changed: replaced multiple values by others, displaying titles
        expected = """<div class="InlineDiff">
    <div class="diff_sub">First Title</div>
    <div class="diff_add"></div>
</div>
<div class="InlineDiff">second_value</div>
<div class="InlineDiff">
    <div class="diff_sub"></div>
    <div class="diff_add">Third Title</div>
</div>"""
        self._test_diff_list(
            [testing.VOCABULARY_TUPLES[0][0], testing.VOCABULARY_TUPLES[1][0]],
            [testing.VOCABULARY_TUPLES[1][0], testing.VOCABULARY_TUPLES[2][0]],
            False,
            expected,
        )
        # changed: replaced multiple values by others, displaying titles
        expected = """<div class="InlineDiff">
    <div class="diff_sub"></div>
    <div class="diff_add">Third Title</div>
</div>
<div class="InlineDiff">First Title</div>
<div class="InlineDiff">
    <div class="diff_sub">second_value</div>
    <div class="diff_add"></div>
</div>"""
        self._test_diff_list(
            [testing.VOCABULARY_TUPLES[0][0], testing.VOCABULARY_TUPLES[1][0]],
            [testing.VOCABULARY_TUPLES[2][0], testing.VOCABULARY_TUPLES[0][0]],
            False,
            expected,
        )
        # changed: removed values, displaying titles
        expected = """<div class="InlineDiff">
    <div class="diff_sub">First Title</div>
    <div class="diff_add"></div>
</div>
<div class="InlineDiff">
    <div class="diff_sub">second_value</div>
    <div class="diff_add"></div>
</div>"""
        self._test_diff_list(
            [testing.VOCABULARY_TUPLES[0][0], testing.VOCABULARY_TUPLES[1][0]],
            [],
            False,
            expected,
        )

    def _test_diff_list(self, value1, value2, same, expected):
        self.obj1.choices = value1
        self.obj2.choices = value2
        diff = ListDiff(self.obj1, self.obj2, "choices")
        self.assertEqual(diff.same, same)
        self.assertEqual(diff.inline_diff(), expected)

    def testGetLineDictDiffsSame(self):
        """test getLineDiffs() method with dict same value"""
        c = C()
        diff = ListDiff(c, c, "attribute")
        expected = [("equal", 0, 1, 0, 1)]
        self.assertEqual(diff.getLineDiffs(), expected)

    def testGetLineDictDiffsDifferent(self):
        """test getLineDiffs() method with dict different value"""
        c = C()
        d = D()
        diff = ListDiff(c, d, "attribute")
        expected = [("replace", 0, 1, 0, 1)]
        self.assertEqual(diff.getLineDiffs(), expected)



#
# CMFDiffTool tests
#
from os import linesep
from plone.app.testing import PLONE_INTEGRATION_TESTING
from Products.CMFDiffTool.FieldDiff import dump
from Products.CMFDiffTool.FieldDiff import FieldDiff
from unittest import TestCase


_marker = []


class A:
    attribute = "value"

    def method(self):
        return "method value"


class B:
    attribute = "different value"

    def method(self):
        return "different method value"


class U:
    attribute = "\xfcnicode value"

    def method(self):
        return "different method val\xfce"


class H:
    attribute = '<script>alert("Hacker value")</script>'

    def method(self):
        return '<script>alert("Hacker method value")</script>'


class TestFieldDiff(TestCase):
    """Test the FieldDiff class"""

    layer = PLONE_INTEGRATION_TESTING

    def testInterface(self):
        """Ensure that tool instances implement the portal_diff interface"""
        from Products.CMFDiffTool.interfaces.portal_diff import IDifference

        self.assertTrue(IDifference.implementedBy(FieldDiff))

    def testAttributeSame(self):
        """Test attribute with same value"""
        a = A()
        fd = FieldDiff(a, a, "attribute")
        self.assertTrue(fd.same)
        uu = U()
        fd = FieldDiff(uu, uu, "attribute")
        self.assertTrue(fd.same)

    def testMethodSame(self):
        """Test method with same value"""
        a = A()
        fd = FieldDiff(a, a, "method")
        self.assertTrue(fd.same)
        uu = U()
        fd = FieldDiff(uu, uu, "method")
        self.assertTrue(fd.same)

    def testAttributeDiff(self):
        """Test attribute with different value"""
        a = A()
        b = B()
        uu = U()
        fd = FieldDiff(a, b, "attribute")
        self.assertFalse(fd.same)
        fd = FieldDiff(a, uu, "attribute")
        self.assertFalse(fd.same)

    def testMethodDiff(self):
        """Test method with different value"""
        a = A()
        b = B()
        uu = U()
        fd = FieldDiff(a, b, "method")
        self.assertFalse(fd.same)
        fd = FieldDiff(a, uu, "method")
        self.assertFalse(fd.same)

    def testGetLineDiffsSame(self):
        """test getLineDiffs() method with same value"""
        a = A()
        fd = FieldDiff(a, a, "attribute")
        expected = [("equal", 0, 1, 0, 1)]
        self.assertEqual(fd.getLineDiffs(), expected)
        uu = U()
        fd = FieldDiff(uu, uu, "attribute")
        expected = [("equal", 0, 1, 0, 1)]
        self.assertEqual(fd.getLineDiffs(), expected)

    def testGetLineDiffsDifferent(self):
        """test getLineDiffs() method with different value"""
        a = A()
        b = B()
        uu = U()
        fd = FieldDiff(a, b, "attribute")
        expected = [("replace", 0, 1, 0, 1)]
        self.assertEqual(fd.getLineDiffs(), expected)
        fd = FieldDiff(a, uu, "attribute")
        expected = [("replace", 0, 1, 0, 1)]
        self.assertEqual(fd.getLineDiffs(), expected)

    def testSameText(self):
        """Test text diff output with same value"""
        a = A()
        fd = FieldDiff(a, a, "attribute")
        self.assertEqual(fd.ndiff(), "  value")
        uu = U()
        fd = FieldDiff(uu, uu, "attribute")
        self.assertEqual(fd.ndiff(), "  \xfcnicode value")

    def testDiffText(self):
        """Test text diff output with different value"""
        a = A()
        b = B()
        uu = U()
        expected = "- value%s+ different value" % linesep
        fd = FieldDiff(a, b, "attribute")
        self.assertEqual(fd.ndiff(), expected)
        expected = "- value%s+ \xfcnicode value" % linesep
        fd = FieldDiff(a, uu, "attribute")
        self.assertEqual(fd.ndiff(), expected)

    def test_dump_text(self):
        """Test dumping a diff of text."""
        diff = []
        dump("-", ["support"], 0, 1, diff)
        self.assertEqual(diff, ["- support"])
        # Try unicode, a 'u' with an umlaut.
        diff = []
        dump("+", ["s\xfcpport"], 0, 1, diff)
        self.assertEqual(diff, ["+ s\xfcpport"])
        # Combine them.
        diff = []
        dump("-", ["support"], 0, 1, diff)
        dump("+", ["s\xfcpport"], 0, 1, diff)
        self.assertEqual(diff, ["- support", "+ s\xfcpport"])

    def test_dump_integer(self):
        """Test dumping a diff of integers."""
        diff = []
        dump("-", [4], 0, 1, diff)
        self.assertEqual(diff, ["- 4"])
        dump("+", [42], 0, 1, diff)
        self.assertEqual(diff, ["- 4", "+ 42"])

    def test_dump_float(self):
        """Test dumping a diff of floats."""
        diff = []
        dump("-", [1.1], 0, 1, diff)
        self.assertEqual(diff, ["- 1.1"])
        dump("+", [1.2], 0, 1, diff)
        self.assertEqual(diff, ["- 1.1", "+ 1.2"])

    def test_dump_boolean(self):
        """Test dumping a diff of booleans."""
        diff = []
        dump("-", [True], 0, 1, diff)
        self.assertEqual(diff, ["- True"])
        dump("+", [False], 0, 1, diff)
        self.assertEqual(diff, ["- True", "+ False"])

    def test_inline_diff_same(self):
        """Test inline diff for attribute with same value"""
        a = A()
        uu = U()
        h = H()
        # We mostly just want to check that the inline diff renders without error.
        fd = FieldDiff(a, a, "attribute")
        self.assertIn('class="InlineDiff"', fd.inline_diff())
        fd = FieldDiff(uu, uu, "attribute")
        self.assertIn('class="InlineDiff"', fd.inline_diff())
        self.assertNotIn("&gt;", fd.inline_diff())
        fd = FieldDiff(h, h, "attribute")
        self.assertIn('class="InlineDiff"', fd.inline_diff())
        # h.attribute contains a script, and this should be escaped.
        self.assertNotIn(h.attribute, fd.inline_diff())
        self.assertIn("&gt;", fd.inline_diff())

    def test_inline_diff_different(self):
        """Test inline diff for attribute with different value"""
        a = A()
        uu = U()
        h = H()
        fd = FieldDiff(a, uu, "attribute")
        self.assertIn('class="InlineDiff"', fd.inline_diff())
        fd = FieldDiff(uu, a, "attribute")
        self.assertIn('class="InlineDiff"', fd.inline_diff())
        self.assertNotIn("&gt;", fd.inline_diff())
        fd = FieldDiff(uu, h, "attribute")
        self.assertIn('class="InlineDiff"', fd.inline_diff())
        # h.attribute contains a script, and this should be escaped.
        self.assertNotIn(h.attribute, fd.inline_diff())
        fd = FieldDiff(h, uu, "attribute")
        self.assertIn('class="InlineDiff"', fd.inline_diff())
        # h.attribute contains a script, and this should be escaped.
        self.assertNotIn(h.attribute, fd.inline_diff())
        self.assertIn("&gt;", fd.inline_diff())



from plone.app.testing import PLONE_INTEGRATION_TESTING
from plone.app.textfield.value import RichTextValue
from Products.CMFDiffTool.CMFDTHtmlDiff import CMFDTHtmlDiff
from Products.CMFDiffTool.interfaces import IDifference

import unittest


class DummyType:
    def __init__(self, body):
        self.body = body


class RichTextDiffTestCase(unittest.TestCase):
    """Test RichTextDiff"""

    layer = PLONE_INTEGRATION_TESTING

    def test_parseField_value_is_none(self):
        value = None
        diff = CMFDTHtmlDiff(DummyType(value), DummyType(value), "body")
        self.assertEqual(diff._parseField(value), [])

    def test_parseField_value_is_not_none(self):
        value = RichTextValue("foo")
        diff = CMFDTHtmlDiff(DummyType(value), DummyType(value), "body")
        self.assertEqual(diff._parseField(value), ["foo"])

    def test_inline_diff_same(self):
        value = RichTextValue("foo")
        diff = CMFDTHtmlDiff(DummyType(value), DummyType(value), "body")
        inline_diff = diff.inline_diff()

        self.assertTrue(IDifference.providedBy(diff))
        self.assertEqual(diff.same, True)
        self.assertEqual(inline_diff, "foo ")

    def test_inline_diff_same_hacker(self):
        value = RichTextValue('<script>alert("Hacker value")</script>')
        diff = CMFDTHtmlDiff(DummyType(value), DummyType(value), "body")
        inline_diff = diff.inline_diff()
        # The script tag should not be escaped, but totally not shown.
        self.assertNotIn("<script", inline_diff)
        self.assertNotIn("&gt;", inline_diff)

    def test_inline_diff_different(self):
        old_value = RichTextValue("foo")
        new_value = RichTextValue("foo bar")
        diff = CMFDTHtmlDiff(DummyType(old_value), DummyType(new_value), "body")

        inline_diff = diff.inline_diff()

        self.assertTrue(IDifference.providedBy(diff))
        self.assertEqual(diff.same, False)
        self.assertEqual(inline_diff, 'foo <span class="insert">bar </span> ')

    def test_inline_diff_different_hacker(self):
        old_value = RichTextValue("clean")
        new_value = RichTextValue('<script>alert("Hacker value")</script>')
        diff = CMFDTHtmlDiff(DummyType(old_value), DummyType(new_value), "body")
        inline_diff = diff.inline_diff()
        # The script tag should not be escaped, but totally not shown.
        self.assertNotIn("<script", inline_diff)
        self.assertNotIn("&gt;", inline_diff)

        old_value = RichTextValue('<script>alert("Hacker value")</script>')
        new_value = RichTextValue("clean")
        diff = CMFDTHtmlDiff(DummyType(old_value), DummyType(new_value), "body")
        inline_diff = diff.inline_diff()
        # The script tag should not be escaped, but totally not shown.
        self.assertNotIn("<script", inline_diff)
        self.assertNotIn("&gt;", inline_diff)



from Products.CMFDiffTool.testing import CMFDiffToolDXLayer

import unittest


class BaseDXTestCase(unittest.TestCase):
    """A base testing class for CMFDiffTool

    It includes a layer which installs the product
    and some testing dependencies in a Plone site.
    """

    layer = CMFDiffToolDXLayer

    def setUp(self):
        self.portal = self.layer["portal"]
